-- Demo1 Setup
-- Create the database and take a full backup

USE master;
GO

IF DATABASEPROPERTY ('ZerodPageExample', 'Version') > 0
	DROP DATABASE ZerodPageExample;
GO

CREATE DATABASE ZerodPageExample;
GO

-- Set the recovery mode to full
ALTER DATABASE ZerodPageExample SET RECOVERY FULL;
GO

-- Create a table to play with.
CREATE TABLE ZerodPageExample.dbo.sales (
	salesID INT IDENTITY,
	customerID INT DEFAULT CONVERT (INT, 100000 * RAND ()),
	salesDate DATETIME DEFAULT GETDATE (),
	salesAmount MONEY);

CREATE CLUSTERED INDEX salesCI ON
	ZerodPageExample.dbo.sales (salesID);
GO

-- Populate the table
SET NOCOUNT ON;
GO

DECLARE @count INT
SELECT @count = 0
WHILE (@count < 5000)
BEGIN
	INSERT INTO ZerodPageExample.dbo.sales (salesAmount)
		VALUES (100 * RAND ());
	SELECT @count = @count + 1
END;
GO

-- Take a full backup.
BACKUP DATABASE ZerodPageExample TO
	DISK = 'C:\SQLskills\ZerodPageExample.bak'
	WITH INIT;
GO
BACKUP LOG ZerodPageExample TO
	DISK = 'C:\SQLskills\ZerodPageExample_LOG1.bak'
	WITH INIT;
GO

-- Now shutdown and zero out a page to simulate an IO error
-- Picking page (1, 158) to corrupt
-- Remember to then backup the corrupt database for Part 2
/*
BACKUP DATABASE ZerodPageExample TO
	DISK = 'C:\SQLskills\ZerodPageExample_Corrupt.bak';
	WITH INIT;
GO
*/