-- Demo1: Disaster recovery simulation of a zero'd page
-- Part 1: Using single page restore

-- Must execute setup + corrupt the database first for this to work.

-- Innocent user query on the sales table
SELECT SUM (salesAmount) FROM ZerodPageExample.dbo.sales
	WHERE salesAmount < $1.00;
GO

-- Uh-oh - corruption!
-- Check the database using DBCC CHECKDB
DBCC CHECKDB (ZerodPageExample)
	WITH ALL_ERRORMSGS, NO_INFOMSGS;
GO

-- What does the page look like? Try doing an
-- interpreted dump of the page using DBCC PAGE
DBCC TRACEON (3604); -- necessary for correct output
GO
DBCC PAGE (ZerodPageExample, 1, 158, 3);
GO

-- DBCC PAGE can't work out how to dump the page
-- because the header is zero'd. Try a hex dump.
DBCC PAGE (ZerodPageExample, 1, 158, 2);
GO

-- Wow - totally zero! Looks like an IO error.

-- First thing to do, disallow user access.
-- Remember not to set it to OFFLINE otherwise not
-- even sysadmin can access the database.
ALTER DATABASE ZerodPageExample SET RESTRICTED_USER;
GO

-- Before we decide to fix the database using single-page
-- restore, let's see if any other pages are corrupt
SELECT * FROM msdb..suspect_pages;
GO

-- Now restore the single damaged page
USE master;
GO

-- First the full backup...
RESTORE DATABASE ZerodPageExample
	PAGE = '1:158' FROM
	DISK = 'C:\SQLskills\ZerodPageExample.bak';
GO

-- Now the first log backup...
RESTORE LOG ZerodPageExample FROM
	DISK = 'C:\SQLskills\ZerodPageExample_LOG1.bak'
	WITH NORECOVERY;
GO
-- And so on...

-- Now the backup and restore the tail of the log...
BACKUP LOG ZerodPageExample TO
	DISK = 'C:\SQLskills\ZerodPageExample_LOG_TAIL.bak'
	WITH INIT;
GO
RESTORE LOG ZerodPageExample FROM
	DISK = 'C:\SQLskills\ZerodPageExample_LOG_TAIL.bak'
	WITH RECOVERY;
GO

-- Bring the database online again
ALTER DATABASE ZerodPageExample SET MULTI_USER;
GO

-- Check the database for corruption again
DBCC CHECKDB (ZerodPageExample)
	WITH ALL_ERRORMSGS, NO_INFOMSGS;
GO

-- Final sanity check - run the user query again
SELECT SUM (salesAmount) FROM ZerodPageExample.dbo.sales
	WHERE salesAmount < $1.00;
GO
