-- Demo1: Disaster recovery simulation of a zero'd page
-- Part 2: Not able to use page restore

-- Now imagine we only had the full backup and
-- the database is in SIMPLE recovery mode.
-- No single page restore is possible.

/*
RESTORE DATABASE ZerodPageExample FROM
	DISK = 'C:\SQLskills\ZerodPageExample_Corrupt.bak'
	WITH REPLACE;
GO
*/

-- Innocent user query on the sales table
SELECT SUM (salesAmount) FROM ZerodPageExample.dbo.sales
	WHERE salesAmount < $1.00;
GO

-- Uh-oh - corruption!
-- Check the database using DBCC CHECKDB
DBCC CHECKDB (ZerodPageExample)
	WITH ALL_ERRORMSGS, NO_INFOMSGS;
GO

-- Hmm - only a full backup. What to do?
-- We're going to have to run repair to fix this.
-- Can we use the full backup to get the data back?

-- First thing - take a backup of the corrupt database
-- in case anything goes wrong.
BACKUP DATABASE ZerodPageExample TO
	DISK = 'C:\SQLskills\ZerodPageExample_BeforeRepair.bak'
	WITH INIT;
GO

-- Now run repair
ALTER DATABASE ZerodPageExample SET SINGLE_USER;
GO

DBCC CHECKDB (ZerodPageExample, REPAIR_ALLOW_DATA_LOSS)
	WITH ALL_ERRORMSGS, NO_INFOMSGS;
GO

ALTER DATABASE ZerodPageExample SET RESTRICTED_USER;
GO

-- How many rows did we lose?
SELECT COUNT (*) FROM ZerodPageExample.dbo.sales;
GO

-- Can we tell the range?
-- Start of the missing range
SELECT MIN(salesID + 1) FROM ZerodPageExample.dbo.sales as A
WHERE NOT EXISTS (
	SELECT salesID FROM ZerodPageExample.dbo.sales as B
	WHERE B.salesID = A.salesID + 1);
GO
-- End of the missing range
SELECT MAX(salesID - 1) FROM ZerodPageExample.dbo.sales as A
WHERE NOT EXISTS (
	SELECT salesID FROM ZerodPageExample.dbo.sales as B
	WHERE B.salesID = A.salesID - 1)
GO








-- Looks like salesID 1226 to 1470 inclusive

-- Let's restore the old full backup to a different location
RESTORE FILELISTONLY FROM
	DISK = 'C:\SQLskills\ZerodPageExample.bak';
GO

RESTORE DATABASE ZerodPageExampleCopy FROM
	DISK = 'C:\SQLskills\ZerodPageExample.bak'
	WITH MOVE N'ZerodPageExample'
		TO N'C:\SQLskills\ZerodPageExampleCopy.mdf', 
	MOVE N'ZerodPageExample_log'
		TO N'C:\SQLskills\ZerodPageExampleCopy_log.ldf', 
	REPLACE;
GO

-- And look in the corrupt page
DBCC TRACEON (3604);
GO
DBCC PAGE (ZerodPageExampleCopy, 1, 158, 3);
GO

-- Cool! The page in the backup contains the
-- data range that is missing. Now we can copy it
-- back to the real database

-- Preserve identity values
SET IDENTITY_INSERT ZerodPageExample.dbo.sales ON;
GO

-- Copy the rows over
SET NOCOUNT OFF;
GO

INSERT INTO ZerodPageExample.dbo.sales (
	salesID, customerID, salesDate, salesAmount)
SELECT * FROM ZerodPageExampleCopy.dbo.sales AS copy
WHERE copy.salesID > 1225 AND copy.salesID < 1471;
GO

-- Restore identity behavior.
SET IDENTITY_INSERT ZerodPageExample.dbo.sales OFF;
GO

-- Check the row count
SELECT COUNT(*) FROM ZerodPageExample.dbo.sales;
GO


